if not AceLibrary:HasInstance("PratPlugin-1.0") then
    DreamChat = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
else
    DreamChat = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "PratPlugin-1.0")
    DreamChat.PratEnabled=1
end    
BINDING_HEADER_DREAMCHAT="DreamChat"

-- Set to a function which will provide timestamps
DreamChat.timestampProvider = nil

-- @desc Initializes the addon (AceAddon).
function DreamChat:OnInitialize()
	self:RegisterDB("DreamChatDB")
	self.frames, self.numframes = {}, 0
	
	self:InitializeMessaging()
	if self.db.profile.colors==nil then
		self.db.profile.colors={}
	end
	if self.db.profile.numframes==nil then
		local f=self:CreateChatFrame("DreamChat")
		if f.db.profile.acceptedTypes=={} then
			f:SetAcceptsAllMessageTypes(true)
		end
	else
		local i,f;
		for i=1,self.db.profile.numframes do
			self:CreateChatFrame("DreamChat")
		end
		-- After creating frames, let's make sure they are laid out correctly.
		for i,f in ipairs(self.frames) do
			f:RefreshLayout()
		end
	end
	self.numframes=self.db.profile.numframes
	self:InitializeEditBox()
	--self:SetDebugging(true)
end

-- @desc The Lazy Man's AceDebug.
function DreamChat:SetDebugging(enabled)
	self.isDebugging=enabled or nil
end
-- @desc The Lazy Man's AceDebug.
function DreamChat:Debug(...)
	if self.isDebugging then
		DEFAULT_CHAT_FRAME:AddMessage("DC Debug: "..string.format(...),1,0,1)
	end
end


-- @desc Creates a new chat frame.
-- @ret object Returns the new chat frame.
function DreamChat:CreateChatFrame(desc)
	self.numframes = self.numframes + 1
	self.db.profile.numframes=self.numframes
	self.frames[self.numframes] = DreamChat.FrameClass:new(self.numframes, desc)
	return self.frames[self.numframes]
end

-- @desc Cosine interpolation.
-- @arg y1 number Start Value
-- @arg y2 number End Value
-- @arg mu number Step Position (0..1)
-- @cat Util
local function CosineInterpolate(y1, y2, mu)
	local mu2 = (1 - math.cos(mu*math.pi)) / 2
	return y1*(1-mu2)+y2*mu2
end

-- @desc Starts interpolating a frame height for the specified DreamChat frame.
-- @arg frame object The frame object.
-- @arg height number The desired frame height.
function DreamChat:SetFrameHeight(frame, height)
	local oldheight, steps = frame:GetHeight(), 15
	frame.heightc = 0
	self:ScheduleRepeatingEvent("DC_FrameHeight"..frame.class.id, self.SetFrameHeight_Handler, 0.5/steps, self, frame, oldheight, height, steps)
end

-- @desc Worker function for setting the frame size. Called by AceEvent as a repeating scheduled event.
-- @arg frame object The frame object.
-- @arg oh number Old height.
-- @arg h number Desired height.
-- @arg steps Number of steps in the operation.
function DreamChat:SetFrameHeight_Handler(frame, oh, h, steps)
	frame.heightc = frame.heightc + 1
	frame:SetHeight(CosineInterpolate(oh, h, ((1/steps) * frame.heightc)))
	if frame.heightc == steps then
		self:CancelScheduledEvent("DC_FrameHeight"..frame.class.id)
		frame.heightc = nil
	end
end

-- @desc Makes a name a clickable link. May also colorize it.
-- @arg name string The name.
-- @ret string The name, linkified.
-- @cat Util
function DreamChat:LinkifyName(name)
	local cdname;
	if TCM_ColorizeName then -- topChatMod support. (topChatMod is a chat mod I (AKX) made for pretty much private use. :)
		cdname=TCM_ColorizeName(name);
	elseif SCCN_ForgottenChatNickName~=nil and SCCN_colornicks == 1 then -- SCCN support.
		cdname=SCCN_ForgottenChatNickName(name);
	elseif self.nameColorizer~=nil then -- external hook
		cdname=self.nameColorizer(name);
	else
		cdname=name;
	end
	return "|Hplayer:"..name.."|h"..cdname.."|h";
end

-- @desc Renders a sender name.
-- @arg message table The message structure.
-- @ret string The name.
-- @cat Util
function DreamChat:RenderSender(message)
	local str="";
	if message.status~=nil and message.status~="" then
		str="<"..(message.status)..">";
	end
	if not message.noBrackets then
		str=str.."["..DreamChat:LinkifyName(message.sender).."]";
	else
		str=str..DreamChat:LinkifyName(message.sender);
	end
	return str;
end

-- @desc Utility function to add a language prefix to a message if needed
-- @arg message table The message structure.
-- @ret string The possibly languagized string.
-- @cat Util
function DreamChat:Languagize(message)
	if not (message.language==nil or message.language=="" or GetDefaultLanguage("player")==message.language) then
		return "["..message.language.."] "..message.text
	end
	return message.text
end

DreamChat.HexMemo={}
-- @desc Makes a hex triplet three rgb(0..1) values with memoization. No sanity checking.
-- @arg hex string The triplet.
-- @ret table The values in a table.
-- @cat Util
function DreamChat:HexToRGB(hex)
	if DreamChat.HexMemo[hex]~=nil then
		return unpack(DreamChat.HexMemo[hex])
	end
	local r,g,b;
	
	r=tonumber(string.sub(hex,1,2),16)/255.0;
	g=tonumber(string.sub(hex,3,4),16)/255.0;
	b=tonumber(string.sub(hex,5,6),16)/255.0;
	DreamChat.HexMemo[hex]={r,g,b};
	return r,g,b;
end