local dewdrop = AceLibrary("Dewdrop-2.0")

-- @desc Does some initialization for the edit box
function DreamChat:InitializeEditBox()
	self.editBoxMsgType="say"
	self.editBoxParam=""
	self:UpdateEditBoxHeader()
	if self.db.profile.editBoxPosition==nil then
		self.db.profile.editBoxPosition={1,"TOPLEFT","BOTTOMLEFT",0,0,"TOPRIGHT","BOTTOMRIGHT",0,0,-1,32}
	end
	self:PositionEditBox()
end

function DreamChat:ResetEditBoxPosition()
	self.db.profile.editBoxPosition={1,"TOPLEFT","BOTTOMLEFT",0,0,"TOPRIGHT","BOTTOMRIGHT",0,0,-1,32}
	self:PositionEditBox()
end

function DreamChat:PositionEditBox()
	local frameId,ap1,at1,x1,y1,ap2,at2,x2,y2,w,h=unpack(self.db.profile.editBoxPosition);
	local frame;
	if frameId>0 then
		if self.frames[frameId] then
			frame=self.frames[frameId].frame;
		else
			frame=self.frames[1].frame;
		end
	end
	if frame==nil then
		frame=UIParent;
	end
	DreamChatEditBox:ClearAllPoints()
	if ap1==0 and ap2==0 then
		DreamChatEditBox:SetPoint("BOTTOMLEFT",frame,"BOTTOMLEFT",0,0)
		DreamChatEditBox:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0)
		DreamChatEditBox:SetHeight(32)
	else
		if ap1~=0 and at1 then
			DreamChatEditBox:SetPoint(ap1,frame,at1,x1,y1)
		end
		if ap2~=0 and at2 then
			DreamChatEditBox:SetPoint(ap2,frame,at2,x2,y2)
		end
		if w and w>0 then DreamChatEditBox:SetWidth(w) end
	end
	DreamChatEditBox:SetHeight(32)
end


-- @desc Shows or hides the edit box, depending on the state parameter.
-- @arg state boolean For any non-nil value, show.
function DreamChat:SetEditBoxShown(state)
	if state then
		if not self.language and GetDefaultLanguage("player") then
			self.language=string.upper(GetDefaultLanguage("player"))
		end
		DreamChatEditBox:Show()
		DreamChatEditBox:SetFocus()
		DreamChat:UpdateEditBoxHeader()
	else
		DreamChatEditBox:Hide()
		DreamChatEditBox:ClearFocus()
	end
end

function DreamChat:OpenRetell()
	if DreamChat.lastWhispered~=nil then
		self.editBoxParam=DreamChat.lastWhispered
		self:SetEditBoxMessageType("whisper")
		self:ShowEditBox()
	end
end
function DreamChat:OpenReply()
	if DreamChat.lastWhisperer~=nil then
		self.editBoxParam=DreamChat.lastWhisperer
		self:SetEditBoxMessageType("whisper")
		self:ShowEditBox()
	end
end

-- @desc Returns whether the edit box is shown.
-- @ret boolean Is the editbox shown?
function DreamChat:IsEditBoxShown()
	return DreamChatEditBox:IsShown()
end

-- @desc Hides the edit box.
function DreamChat:HideEditBox()
	return self:SetEditBoxShown(nil)
end

-- @desc Shows the edit box.
function DreamChat:ShowEditBox()
	return self:SetEditBoxShown(1)
end

-- @desc Toggles the edit box.
function DreamChat:ToggleEditBox()
	return self:SetEditBoxShown(not self:IsEditBoxShown())
end

-- @desc Opens the editbox menu.
function DreamChat:OpenEditBoxMenu()
	dewdrop:Open(this,'children',function(l,v,v1,v2,v3,v4)self:FeedEditBoxDewdrop(l,v,v1,v2,v3,v4)end)
end

function DreamChat:FeedEditBoxLanguageDewdrop(l,v,v1,v2,v3,v4)
	local _,v,known,r,g,b;
	known={};
	for _=1, GetNumLaguages() do
	  known[GetLanguageByIndex(_)]=true
	end
	for _,v in ipairs(DreamChat.languages) do

		if v==GetDefaultLanguage("player") or DreamChat.languageNames[v]==GetDefaultLanguage("player") then
			r,g,b=1,0.8,0;
		elseif known[v] or known[DreamChat.languageNames[v]] then
			r,g,b=1,1,1;
		else
			r,g,b=0.85,0.85,0.85;
		end
		dewdrop:AddLine(	'text', DreamChat.languageNames[v],
					'arg1',v,
					'checked',(DreamChat.language==v),
					'func',function(a)self:SetLanguage(a);dewdrop:Refresh(2);DreamChat:UpdateEditBoxHeader()end,
					'textR',r,'textG',g,'textB',b)
	end
end

local attachmentPoints=
	{
		{	0,		"Disable Attachment",	},
		{	"TOPLEFT",	"Upper Left",		},
		{	"TOPRIGHT",	"Upper Right",		},
		{	"TOP",		"Upper Center",		},
		{	"BOTTOMLEFT",	"Bottom Left",		},
		{	"BOTTOMRIGHT",	"Bottom Right",		},
		{	"BOTTOM",	"Bottom Center",	},
		{	"LEFT",		"Left",			},
		{	"RIGHT",	"Right",		},
		{	"CENTER",	"Center",		}
	}

function DreamChat:FeedEditBoxDewdrop(l,v,v1,v2,v3,v4)
	local o;
	if l==1 then
		dewdrop:AddLine('text', 'Language','hasArrow',true,'value','language')
		dewdrop:AddLine('text', 'Position','hasArrow',true,'value','position')
	elseif l==2 then
		if v=="language" then
			self:FeedEditBoxLanguageDewdrop(l,v,v1,v2,v3,v4)
		end
		if v=="position" then
			
			dewdrop:AddLine('text','Attach to...','hasArrow',true,'value','attachTo')
			dewdrop:AddLine('text','Presets...','hasArrow',true,'value','presets')
			dewdrop:AddLine('text','Attachment 1...','hasArrow',true,'value','attach1')
			dewdrop:AddLine('text','Attachment 2...','hasArrow',true,'value','attach2')
			dewdrop:AddLine('text','Width','hasArrow',true,'hasSlider', true, 'sliderMin', -1, 'sliderMax', 800, 'sliderValue', self.db.profile.editBoxPosition[10],'sliderFunc',function(v)self.db.profile.editBoxPosition[10]=v;self:PositionEditBox()end)
			--dewdrop:AddLine('text','Height','hasArrow',true,'hasSlider', true, 'sliderMin', -1, 'sliderMax', 800, 'sliderValue', self.db.profile.editBoxPosition[11],'sliderFunc',function(v)self.db.profile.editBoxPosition[11]=v;self:PositionEditBox()end)
		end
	elseif l==3 then
		if v1=="position" then
			if v=="presets" then
				dewdrop:AddLine('text','Top','func',
					function()
						self.db.profile.editBoxPosition={self.db.profile.editBoxPosition[1],"BOTTOMLEFT","TOPLEFT",0,0,"BOTTOMRIGHT","TOPRIGHT",0,0,-1,32};
						self:PositionEditBox()
					end
				);
				dewdrop:AddLine('text','Bottom','func',
					function()
						self.db.profile.editBoxPosition={self.db.profile.editBoxPosition[1],"TOPLEFT","BOTTOMLEFT",0,0,"TOPRIGHT","BOTTOMRIGHT",0,0,-1,32};
						self:PositionEditBox()
					end
				);
				dewdrop:AddLine('text','Bottom (inside)','func',
					function()
						self.db.profile.editBoxPosition={self.db.profile.editBoxPosition[1],"BOTTOMLEFT","BOTTOMLEFT",0,0,"BOTTOMRIGHT","BOTTOMRIGHT",0,0,-1,32};
						self:PositionEditBox()
					end
				);
				
			end

			if v=="attachTo" then
				dewdrop:AddLine('text','Floating','checked',(self.db.profile.editBoxPosition[1]<1),'func',function()self.db.profile.editBoxPosition[1]=0;self:PositionEditBox()end)
				for i=1,self.numframes do
					dewdrop:AddLine('text',string.format("[%d] %s...",i, self.frames[i].db.profile.title),'checked',(self.db.profile.editBoxPosition[1]==i),'func',function()self.db.profile.editBoxPosition[1]=i;self:PositionEditBox()end)
				end
			end
			if v=="attach1" or v=="attach2" then
				if v=="attach1" then
					o=2;
				else
					o=6;
				end
				dewdrop:AddLine('text','Attach Point...','hasArrow',true,'value','ap')
				dewdrop:AddLine('text','Attach To...','hasArrow',true,'value','at')
				dewdrop:AddLine('text','X Offset','hasArrow',true,'hasSlider', true, 'sliderMin', -800, 'sliderMax', 800, 'sliderValue', self.db.profile.editBoxPosition[o+2],'sliderFunc',function(v)self.db.profile.editBoxPosition[o+2]=v;self:PositionEditBox()end)
				dewdrop:AddLine('text','Y Offset','hasArrow',true,'hasSlider', true, 'sliderMin', -800, 'sliderMax', 800, 'sliderValue', self.db.profile.editBoxPosition[o+3],'sliderFunc',function(v)self.db.profile.editBoxPosition[o+3]=v;self:PositionEditBox()end)
			end
		end
	elseif l==4 then
		if v1=="attach1" or v1=="attach2" then
			if v1=="attach1" then
				o=2;
			else
				o=6;
			end
			if v=="ap" or v=="at" then
				local _,p;
				if v=="at" then o=o+1 end;
				for _,p in ipairs(attachmentPoints) do
					dewdrop:AddLine('text',p[2],'checked',(self.db.profile.editBoxPosition[o]==p[1]),'arg1',p[1],'func',function(v)self.db.profile.editBoxPosition[o]=v;self:PositionEditBox()end)
				end
			end
		end
	end
end

-- @desc Called by the widget when the editbox is shown.
-- @cat EventHandler
function DreamChat:OnEditBoxShow()

end

-- @desc Called by the widget when Enter is pressed in the editbox.
-- @cat EventHandler
function DreamChat:EditBoxEnter()
	local text=DreamChatEditBox:GetText();
	local command,msg,r;
	if text~="" then 
		self:AddEditBoxHistory(text);
		if DreamChat.messageTypes[self.editBoxMsgType].nonSticky==true then
			self.editBoxMsgType=self.editBoxLastMsgType;
		end
		DreamChatEditBox:SetText("");
		self:HideEditBox()
		command,msg=self:SplitSlashCmd(text);
		if command~=nil then
			if DreamChat:ExecuteSlashCmd(command,msg)~=nil then return -1; end
			if DreamChat:ExecuteEmoteCmd(command,msg)~=nil then return -2; end
			DreamChat:ShowMessage("dc","The slash command was not recognized.")
			return nil
		end
		r=self:SendMessage(self.editBoxMsgType,text);
		if r==nil then
			DreamChat:ShowMessage("dc","Sending the message ("..self.editBoxMsgType..") failed.")
		end
		return r
	else
		self:HideEditBox()
		return nil;
	end
end

-- @desc Called by the widget when Esc is pressed in the editbox.
-- @cat EventHandler
function DreamChat:EditBoxEscape()
	self:HideEditBox()
end

-- @desc Called by the widget when Space is pressed in the editbox.
-- @cat EventHandler
function DreamChat:EditBoxSpace()

end

-- @desc Called by the widget when Tab is pressed in the editbox.
-- @cat EventHandler
function DreamChat:EditBoxTab()

end

-- @desc Splits a slash command into text and params. Stolen from ChatFrame.lua
-- @arg text string Input text
-- @ret string,string command,parameters
function DreamChat:SplitSlashCmd(text)
	if strsub(text, 1, 1) ~= "/" then return; end
	-- If the string is in the format "/cmd blah", command will be "cmd"
	local command = gsub(text, "/([^%s]+)%s(.*)", "/%1", 1);
	local msg = "";


	if ( command ~= text ) then
		msg = strsub(text, strlen(command) + 2);
	end

	command = gsub(command, "%s+", "");
	command = strupper(command);
	return command,msg;
end

-- @desc Executes the slashcmd function, if any, from the split slash command.
-- @arg command string The COMMAND string.
-- @arg msg string The parameters.
-- @ret string? Returns the slash command identifier if executed, nil if not.
function DreamChat:ExecuteSlashCmd(command,msg)
	local index,value;
	for index, value in pairs(SlashCmdList) do
		local i = 1;
		local cmdString = TEXT(getglobal("SLASH_"..index..i));
		while ( cmdString ) do
			cmdString = string.upper(cmdString);
			if ( cmdString == command ) then
				value(msg);
				return index;
			end
			i = i + 1;
			cmdString = TEXT(getglobal("SLASH_"..index..i));
		end
	end
	return nil;
end

-- @desc Executes the emote function, if any, from the split slash command.
-- @arg command string The COMMAND string.
-- @arg msg string The parameters.
-- @ret string? Returns the emote token if executed, nil if not.
function DreamChat:ExecuteEmoteCmd(command,msg)
	local i = 1;
	local j = 1;
	local cmdString = TEXT(getglobal("EMOTE"..i.."_CMD"..j));
	while ( cmdString ) do
		if ( strupper(cmdString) == command ) then
			local token = getglobal("EMOTE"..i.."_TOKEN");
			if ( token ) then
				DoEmote(token, msg);
			end
			return token;
		end
		j = j + 1;
		cmdString = TEXT(getglobal("EMOTE"..i.."_CMD"..j));
		if ( not cmdString ) then
			i = i + 1;
			j = 1;
			cmdString = TEXT(getglobal("EMOTE"..i.."_CMD"..j));
		end
	end
	return nil;
end

-- @desc Attempts to change the message type from the editbox.
-- @ret boolean 1 if changed.
function DreamChat:SlashMessageTypeChange(text)
	local textl=string.lower(text);
	local k,v,s,command,msg;
	command,msg=DreamChat:SplitSlashCmd(text);
	if command~=nil and command~="" and string.sub(text,-1,-1)==" " then
		for k,v in pairs(DreamChat.slashes) do
			if command==k then
				if DreamChat.messageTypes[v].slashValidator==nil then
					DreamChat:SetEditBoxMessageType(v);
					DreamChatEditBox:SetText(msg);
					return 1;
				else
					s=DreamChat.messageTypes[v].slashValidator(text);
					if s~=nil then
						DreamChat:SetEditBoxMessageType(v);
						DreamChatEditBox:SetText(s);
						return 1;
					end
				end
			end
		end
	end
	return nil;
end

local ignoretc=nil;
local ignorets=nil;

-- @desc Called by the widget when the text in the editbox is changed.
-- @cat EventHandler
function DreamChat:EditBoxChanged()
	if ignoretc then
		ignoretc=nil;
		return;
	end
	local text=this:GetText();
	if text~="" then
		if string.sub(text,1,1)=="/" then
			self:SlashMessageTypeChange(text)
		end
	end
end



-- @desc Called by the widget when the text in the editbox is... set?.
-- @cat EventHandler
function DreamChat:EditBoxTextSet()
	if ignorets then
		ignorets=nil;
		return;
	end
	if string.sub(DreamChatEditBox:GetText(),1,1)=="#" then
		local a,b,mt,mp,txt;
		a,b,mt,mp,txt=strfind(DreamChatEditBox:GetText(),"#(.-)|(.-)|(.*)")
		ignorets=1
		if a~=nil then
			self.editBoxMsgType=mt
			self.editBoxMsgParam=mp
			DreamChatEditBox:SetText(txt)
		end
		DreamChat:UpdateEditBoxHeader()
	else
		DreamChat:EditBoxChanged()
	end
end

-- @desc Sets the editbox message type.
function DreamChat:SetEditBoxMessageType(msgType)
	if DreamChat.messageTypes[msgType] and DreamChat.messageTypes[msgType].sender then
		self.editBoxLastMsgType=self.editBoxMsgType
		self.editBoxMsgType=msgType
		self:UpdateEditBoxHeader()
		return 1
	end
	return nil
end

-- @desc Saves the line in the editbox history buffer. Magically.
function DreamChat:AddEditBoxHistory(text)
	DreamChatEditBox:AddHistoryLine("#"..self.editBoxMsgType.."|"..(self.editBoxParam or "").."|"..text)
end


-- @desc Updates the header line for the editbox.
function DreamChat:UpdateEditBoxHeader()
	local info = DreamChat.messageTypes[self.editBoxMsgType];
	local header = DreamChatEditBoxHeader;
	local htext="";
	local r,g,b;
	if ( not header ) then
		return;
	end
	
	if info.headerText~=nil then
		if type(info.headerText)=="function" then
			htext=info.headerText();
		else
			htext=gsub(info.headerText,"PARAM",self.editBoxParam);
		end
	else
		htext=info.name;
	end
	if GetDefaultLanguage("player")~=nil and self.language~=string.upper(GetDefaultLanguage("player")) and self.language~=nil then
		htext=htext.."|cff888888"..(self.languageNames[self.language] or self.language).."|r";
	end

	header:SetText(htext);
	r,g,b=DreamChat:GetMessageTypeColor(self.editBoxMsgType);
	header:SetTextColor(r, g, b);
	DreamChatEditBox:SetTextInsets(15 + header:GetWidth(), 13, 0, 0);
	DreamChatEditBox:SetTextColor(r, g, b);
end